﻿using System;
using System.Collections.Concurrent;
using System.Threading;
using DTMF_IVR.Model.IVRMenuElements;
using Ozeki.Network.Nat;
using Ozeki.VoIP;
using Ozeki.VoIP.SDK;
using System.Collections.Generic;

namespace DTMF_IVR.Model
{

    #region EngineInformation

    internal class EngineInformation : EventArgs
    {
        public EngineInformation(string engineElementName, string callId, string reportString)
        {
            CallId = callId;
            EngineElementName = engineElementName;
            ReportString = reportString;
        }

        public string CallId { get; set; }
        public string ReportString { get; set; }
        public string EngineElementName { get; set; }
    }

    #endregion

    internal class IVREngine
    {
        private readonly IIVRMenuElement baseIVRMenuElement;
        private IPhoneLine line;

        public Dictionary<IPhoneCall, CallSoundEngine> ActiveCallSoundEngines { get; private set; }

        /// <summary>
        /// Create the IVREngine.
        /// </summary>
        /// <param name="softPhone">Ozeki VoIP SIP SDK softphone</param>
        /// <param name="menuElement"></param>
        /// <param name="acc"></param>
        internal IVREngine(
            Dictionary<IPhoneCall, CallSoundEngine> soundEngines, 
            ISoftPhone softPhone, 
            IIVRMenuElement menuElement, 
            SIPAccount acc)
        {
            SoftPhone = softPhone;
            Acc = acc;
            baseIVRMenuElement = menuElement;
            ActiveCallSoundEngines = soundEngines;
        }

        public ISoftPhone SoftPhone { get; set; }
        public SIPAccount Acc { get; set; }

        public static event EventHandler<EngineInformation> NewEngineEvent;

        /// <summary>
        /// Activates the EngineEvent
        /// </summary>
        /// <param name="elementName"></param>
        /// <param name="callID"></param>
        /// <param name="reportString"></param>
        public static void OnEngineEvent(string elementName, string callID, string reportString)
        {
            if (NewEngineEvent != null)
                NewEngineEvent(null, new EngineInformation(elementName, callID, reportString));
        }


        /// <summary>
        /// Start the Engine.
        /// </summary>
        public void Start()
        {
            OnEngineEvent(string.Format("- - - Tries to register {1} SIP account.",Acc.UserName,Acc.SIPURL),"","");
            line = SoftPhone.CreatePhoneLine(Acc,new NatConfiguration(NatTraversalMethod.None,""));
            line.PhoneLineStateChanged += Line_PhoneLineInformation;
            SoftPhone.RegisterPhoneLine(line);
            OnEngineEvent("- - - IVR service start", "-", "");
        }

        /// <summary>
        /// Handles the events PhoneLineInformation.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Line_PhoneLineInformation(object sender, VoIPEventArgs<PhoneLineState> e)
        {
            if (sender == null)
                return;

            if (e.Item != PhoneLineState.NoRegNeeded && e.Item != PhoneLineState.RegistrationSucceeded)
                return;

            SoftPhone.IncomingCall += SoftPhone_IncomingCall;
        }

        /// <summary>
        /// Handles the events IncommingCall.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void SoftPhone_IncomingCall(object sender, VoIPEventArgs<IPhoneCall> e)
        {
            IPhoneCall call = e.Item;
            CallSoundEngine soundEngine = new CallSoundEngine(call);
            ActiveCallSoundEngines.Add(call, soundEngine);
            call.CallStateChanged += new EventHandler<VoIPEventArgs<CallState>>(call_CallStateChanged);
            call.Accept();
            
            IIVRMenuElement ownMenu = (IIVRMenuElement)baseIVRMenuElement.Clone();
            new Thread(() => ownMenu.EngineExecute(e.Item)).Start();
        }


        void call_CallStateChanged(object sender, VoIPEventArgs<CallState> e)
        {
            var call = sender as IPhoneCall;
            if (call != null)
                if (e.Item.IsCallEnded() && ActiveCallSoundEngines.ContainsKey(call))
                    ActiveCallSoundEngines.Remove(call);
        }

        /// <summary>
        /// Stop the Engine.
        /// </summary>
        internal void Stop()
        {
            SoftPhone.UnregisterPhoneLine(line);
            line.PhoneLineStateChanged -= Line_PhoneLineInformation;
            OnEngineEvent("- - - IVR service stop", "-", "");
        }
    }
}