﻿using System;
using System.Collections.Concurrent;
using System.IO;
using System.Linq;
using System.Net;
using System.Net.Mail;
using System.Threading;
using DTMF_IVR.Model.InputInterpreter;
using Ozeki.Media.MediaHandlers;
using Ozeki.VoIP;
using System.Collections.Generic;

namespace DTMF_IVR.Model.IVRMenuElements
{

    #region VoiceMessageData

    internal class VoiceMessageData
    {
        public VoiceMessageData(string mailFrom, string mailTo, string mailSubject = "", string mailBody = "")
        {
            MailFrom = mailFrom;
            MailTo = mailTo;
            MailSubject = mailSubject;
            MailBody = mailBody;
        }

        public string MailFrom { get; set; }
        public string MailTo { get; set; }
        public string MailSubject { get; set; }
        public string MailBody { get; set; }
    }

    #endregion

    internal class VoiceMessageToEmailElement : BaseIVRMenuElement
    {
        private readonly string smtpClientAddress;
        private readonly string tempPath;
        private readonly VoiceMessageData messageData;
        private string tempFilePath;

        private WaveStreamRecorder WaveStreamRecorder;

        public VoiceMessageToEmailElement(
            Dictionary<IPhoneCall, CallSoundEngine> soundEngines, 
            string smtpClientAddress, 
            VoiceMessageData voiceMessageData,
            string tempPath = null,
            IInputInterpreter inputInterpreter = null)
            : base(soundEngines, inputInterpreter)
        {
            messageData = voiceMessageData;
            this.smtpClientAddress = smtpClientAddress;

            this.tempPath = tempPath ?? Path.GetTempPath();
        }


        /// <summary>
        /// DeActivate of the engine.
        /// </summary>
        public override void EngineStop()
        {
            PhoneCall.CallStateChanged -= PhoneCallCallStateChanged;

            IVREngine.OnEngineEvent("VoiceMessageToEmail", PhoneCall.DialInfo.UserName, " Stop");
            SendEmail();
            DeActivateInputInterpreter();
        }

        /// <summary>
        /// Activation of the engine.
        /// </summary>
        /// <param name="phoneCall"></param>
        public override void EngineExecute(IPhoneCall phoneCall)
        {
            PhoneCall = phoneCall;

            if (phoneCall == null || phoneCall.CallState != CallState.InCall)
                return;
            
            tempFilePath = Path.Combine(tempPath,
                                        phoneCall.PhoneLine.PhoneCalls.First().DialInfo.UserName + "_" +
                                        DateTime.Now.ToString("HH_mm_ss_ffff") + ".wav");


            CallSoundEngine engine = GetSoundEngine(phoneCall);
            if (engine != null)
            {
                WaveStreamRecorder = new WaveStreamRecorder(tempFilePath);
                engine.Connector.Connect(engine.IncomingAudioMixer, WaveStreamRecorder);
                WaveStreamRecorder.StartStreaming();
            }
            phoneCall.CallStateChanged += PhoneCallCallStateChanged;

            IVREngine.OnEngineEvent("VoiceMessageToEmail", phoneCall.DialInfo.UserName, " Start");
            ActivateInputInterpreter();
        }

        /// <summary>
        /// Handles the CallStateChanged event.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void PhoneCallCallStateChanged(object sender, VoIPEventArgs<CallState> e)
        {
            if (e.Item != CallState.InCall)
            {
                IPhoneCall call = (IPhoneCall)sender;
                if (WaveStreamRecorder != null)
                {
                    CallSoundEngine engine = GetSoundEngine(call);
                    if (engine != null)
                        engine.Connector.Disconnect(engine.IncomingAudioMixer, WaveStreamRecorder);
                }
                EngineStop();
            }
        }

        /// <summary>
        /// Send email
        /// </summary>
        private void SendEmail()
        {
            try
            {
                Thread t = new Thread(() =>
                                          {
                                              MailMessage message = new MailMessage(messageData.MailFrom,
                                                                                    messageData.MailTo,
                                                                                    messageData.MailSubject,
                                                                                    messageData.MailBody);

                                              message.Attachments.Add(new Attachment(tempFilePath, "audio/x-wav"));

                                              SmtpClient client = new SmtpClient(smtpClientAddress);
                                              client.Credentials = CredentialCache.DefaultNetworkCredentials;
                                              client.Send(message);
                                              IVREngine.OnEngineEvent("VoiceMessageToEmail", PhoneCall.DialInfo.UserName,
                                                                      "Send email to: " + messageData.MailTo);
                                          }) {IsBackground = true};
            }
            catch (Exception exception)
            {
                IVREngine.OnEngineEvent("VoiceMessageToEmail", PhoneCall.DialInfo.UserName, "Email error: " + exception.Message);
            }
        }

        #region Cloning

        public override object Clone()
        {
            BaseIVRMenuElement menuElement = new VoiceMessageToEmailElement(ItsSoundEngines, smtpClientAddress, messageData, tempPath);

            BaseCloning(menuElement);

            return menuElement;
        }

        #endregion
    }
}